'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Autor: Jonas Eberhard www.joseng.de Projektarbeit EST 2010/2011 BKE09       |
'| Datei: Heizungssteuerung.bas                                                |
'| Beschreibung : Firmware der Heizungssteuerung (zustndig fr Brenn -        |
'|                 berwachung, Pelletnachfhrung und Kesseltemperatur         |
'|                                                                             |
'| Version: 1.0                                                                |
'|                                                                             |
'| Fusebits: Lowbyte: 0xFF       Highbyte: 0xD1                                |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Befehle :                                                                   |
'|                                                                             |
'| - Gluehzeit : Zeit Zum "Vorglhen" Der Glhkerze                            |
'| - Zuendzeit : Zeit Zum Entznden Der Pellets Nach Ein Einwurf               |
'| - Zuendstufe : Geblsestufe Zum Znden Der Heizung [%]                      |
'| - Brennstufe : Geblsestufe Beim Brennen Der Heizung(normalbetrieb) [%]     |
'| - Einwurfzeit : Zeit Wie Lang Sich Die Pelletschnecke Dreht                 |
'| Die Zeit Bestimmt Die Menge Der Pellets                                     |
'| - Tempkessel : Gibt Die Kesseltemperatur Aus                                |
'| - Tempbrenner : Gibt Die Brennertemperatur Aus                              |
'| - Zuendtemp : Die Temperatur Ab Die Der Zndvorgang Abgeschlossen Ist       |
'|(die Heizung Brennt)                                                         |
'| - Brenntemp : Normaltemperatur Beim Brennen Der Heizung , Bei Unterschreiten|
'| Werden Pellets Nachgefhrt                                                  |
'| - Restarttemp : Die Temperatur Ab Die Die Heizung Neu Gestartet Wird        |
'| - Minkesseltemp : Minimale Kesseltemperatur                                 |
'| - Maxkesseltemp : Maximale Kesseltemperatur                                 |
'|                                                                             |
'|                                                                             |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Eeprom Belegung(byte Weise) :                                               |
'|                                                                             |
'| 0 : Glhzeit [s] (1 Byte)                                                   |
'| 1 : Zndzeit [s] (1 Byte)                                                   |
'| 2 : Zndstufe [%] (1 Byte)                                                  |
'| 3 : Brennstufe [%] (1 Byte)                                                 |
'| 4 : Einwurfzeit [s] (1 Byte)                                                |
'| 5 : Zndtemperatur [c] (2 Byte)                                            |
'| 7 : Brenntemperatur [c] (2 Byte)                                           |
'| 9 : Restarttemperatur [c] (2 Byte)                                         |
'| 11 : Minkesseltemp [c] (1 Byte)                                            |
'| 12 : Maxkesseltemp [c] (1 Byte)                                            |
'| 13 : Vref [V] (4 Byte)                                                      |
'| 17 : Pellts (Sperrzeit bis zum nchsten Einwurf)                            |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Prinbelegung :                                                              |
'|                                                                             |
'|   Portc.0(adc0) : Temperaturelement(5mV / C)                               |
'|   Portc.1(adc1) : Pt100 Kessel(vin = 1ma * Rpt100)                          |
'|   Portd.0(rxd) : Serielle Schnittstelle Receive                             |
'|   Portd.1(txd) : Serielle Schnittstelle Transmit                            |
'|   Portd.3 : Richtungspin Rs485 Baustein(busrichtung)                        |
'|   Portd.5 : Pumpe                                                           |
'|   Portd.6 : Schnecke                                                        |
'|   Portd.7 : Glhkerze                                                       |
'|   Portb.1 : Geblse                                                         |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Kommandoaufbau :                                                            |
'|                                                                             |
'|   Kommando : "Modulname: Befehl: Wert"                                      |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'Init
$regfile = "m8def.dat"                                      'ATMega 8
$crystal = 16000000                                         '16MHz Quarz
$baud = 19200                                               'Baudrate 19200

$hwstack = 40
$swstack = 40
$framesize = 40

$projecttime = 5


'Funktionsprototypen
Declare Sub Heizung_starten()
Declare Sub Temperaturen_einlesen()
Declare Sub Interpreter()
Declare Sub Heizung_steuern()


'Ein-/Ausgnge
Config Portd.3 = Output                                     'Busrichtung    0= Lesen 1=Schreiben
Config Portb.1 = Output                                     'Geblse
Config Portd.6 = Output                                     'Motor Schnecke
Config Portd.5 = Output                                     'Pumpe
Config Portd.7 = Output                                     'Trigger Glhkerze


'Aliaszuweisungen
Busrichtung Alias Portd.3
Geblaese Alias Portb.1
Schnecke Alias Portd.6
Pumpe Alias Portd.5
Gluehkerze Alias Portd.7


'ADC
Config Adc = Single , Prescaler = Auto
'ADC Kanal 0 und Kanal 1 sind belegt.
'Kanal 0: Temp Element (5mV/C)
'Kanal 1: PT100


'Timer0 (100H)
Config Timer0 = Timer , Prescale = 1024
On Timer0 Timer0_irq
Enable Timer0


'Timer1 (PWM)
Config Timer1 = Pwm , Pwm = 10 , Compare A Pwm = Clear Down , Compare B Pwm = Clear Down , Prescale = 1
'On Timer1 Timer1_irq
Enable Timer1


'Serielle Schnittstelle RS485
Config Print0 = Portd.3 , Mode = Set


'Deklarationen
Dim Tempbrenner As Integer                                  'Temperatur Brenner [C]
Dim Tempkessel As Integer                                   'Temperatur Kessel [C]
Dim Tempdummy As Single                                     'Rechenvariable fr PT100 Korrektur
Dim Adcwert As Integer                                      'Eingelesener Wert des ADCs
Dim I As Integer                                            'Zhlervariable fr mittelung
Dim Tempkorrektur As Single                                 'Rechenvariable zur Korrektur des PT100
Dim Ptohm As Single                                         'Widerstand des PT100
Dim Vref As Single                                          'Referenzspannung der ADCs = 5V
Dim Modulname As String * 10                                'Modulname fr Bus
Dim Gluehzeit As Byte                                       'Zeit zum "Vorglhen" der Glhkerze
Dim Zuendzeit As Byte                                       'Zeit zum Entznden der Pellets nach ein Einwurf
Dim Geblaesestufe As Byte                                   'Aktuelle Geblsestufe Prozent der PWM des Lfters
Dim Zuendstufe As Byte                                      'Geblsestufe zum Znden der Heizung [%]
Dim Brennstufe As Byte                                      'Geblsestufe beim Brennen der Heizung (Normalbetrieb) [%]
Dim Einwurfzeit As Byte                                     'Zeit wie lang sich die PelletSchnecke zum Einwurf der Pellets dreht
Const Timervorgabe0 = 100                                   'Reloadvalue fr PWMTimer (Timer0)
'Const Timervorgabe1 = 3036                                  'Reloadvalue fr Messtimer (Timer1)
'Dim Pwmcount As Integer                                     'Zhlvariable fr PWM
'Dim Pwmcalc As Integer                                      'Rechenvariable fr PWM
'Const Minpuls = 2                                           'Minimale Counts fr PWM Puls, damit der PWM Puls nicht zu kurz wird, kann Motor schaden
Dim Zuendtemp As Integer                                    'Die Temperatur ab die der Zndvorgang abgeschlossen ist (die Heizung brennt)
'Dim Messung As Bit                                          'Freigabevariable fr Messung und Nachfhrung
Dim Brenntemp As Integer                                    'Normalbrenntemperatur, bei unterschreiten werden Pellets nachgefhrt
Dim Restarttemp As Integer                                  'Temperatur ab die die Heizung neu gestartet wird.
Dim Minkesseltemp As Byte                                   'Minimale Kesseltemperatur [C]
Dim Maxkesseltemp As Byte                                   'Maximale Kesseltemperatur [C]
Dim Heizungstabil As Bit                                    'Gibt an ob die Heizung stabil brennt
Dim Buffer As String * 50                                   'Serieller Buffer
Dim Laenge As Byte                                          'Lnge eines Strings (wird universal eingesetzt)
'Dim Char As Byte                                            'Aktuelles Serielles Zeichen
Dim Kommando As String * 15                                 'Aktuelles Kommanddo von Bus
Dim Wert As Single                                          'Parameter zu Kommando
Dim Modul As String * 10                                    'Name des angesprochenen Moduls
Dim Timer0zaehler As Byte                                   'Zhler fr 1 Sekunde
Dim Pellets As Byte                                         'Zhler damit nicht jede Sekunde Pellets eingeworfen werden, wenn die Greze unterschritten ist


'Einstellungen aus EEprom laden
Readeeprom Gluehzeit , 0
Readeeprom Zuendzeit , 1
Readeeprom Zuendstufe , 2
Readeeprom Brennstufe , 3
Readeeprom Einwurfzeit , 4
Readeeprom Zuendtemp , 5
Readeeprom Brenntemp , 7
Readeeprom Restarttemp , 9
Readeeprom Minkesseltemp , 11
Readeeprom Maxkesseltemp , 12
Readeeprom Vref , 13
Readeeprom Pellets , 17


'Startwerte
Reset Schnecke                                              'Alles aus
Reset Pumpe
Reset Gluehkerze
Pwm1a = 0                                                   'Geblaese

Heizungstabil = 0
Modulname = "Heizung: "

Timer0 = Timervorgabe0
'Timer1 = Timervorgabe1

Enable Interrupts


'Heizung starten
'Call Heizung_starten


'Hauptprogramm
Do
  Input Buffer Noecho
  'Input "> " , Buffer
  Call Interpreter
Loop


'Ablauf zum Heizung starten
Sub Heizung_starten()
  'Heizung brennt nicht
  Heizungstabil = 0

  'Glhkerze an
  Set Gluehkerze

  '"Vorglhzeit" abwarten
  Wait Gluehzeit

  'Pellets einwerfen
  Set Schnecke
  Wait Einwurfzeit
  Reset Schnecke
  Wait 1
  Set Schnecke
  Wait Einwurfzeit
  Reset Schnecke

  'Pellets anznden
  Wait Zuendzeit

  'Geblse einschalten
  Pwm1a = 10 * Zuendstufe

  'Auf Temperatur warten
  Do
    Wait 1
  Loop Until Tempbrenner > Zuendtemp

  'Glhkerze aus
  Reset Gluehkerze

  'Geblse niedriger
  Pwm1a = 10 * Brennstufe

  'Heizung brennt
  Heizungstabil = 1
End Sub

'Befehlinterpreter fr Buskommandos
Sub Interpreter()
  Laenge = Len(modulname)
  Modul = Left(buffer , Laenge)
  If Modul = Modulname Then
    'Seriellen String zerlegen
    'Kommando
    Laenge = Len(buffer) - Laenge
    Buffer = Right(buffer , Laenge)
    Laenge = Instr(buffer , " ")
    Laenge = Laenge - 1
    Kommando = Left(buffer , Laenge)
    Kommando = Lcase(kommando)

    'Wert
    Laenge = Len(buffer) - Laenge
    Laenge = Laenge - 1
    Buffer = Right(buffer , Laenge)
    Wert = Val(buffer)

'    'Debugausgabe
'    Print "M " ; Modul
'    Print "K " ; Kommando
'    Print "W " ; Wert

    Select Case Kommando
      Case "gluehzeit":
        If Buffer = "?" Then
          Print "wert: " ; Wert
          Print
          Print Gluehzeit
          Exit Sub
        End If
        Gluehzeit = Wert
        Writeeeprom Gluehzeit , 0
        Print "Neue Gluezeit: " ; Gluehzeit
      Case "zuendzeit":
        Zuendzeit = Wert
        Writeeeprom Zuendzeit , 1
        Print "Neue Zuendzeit: " ; Zuendzeit
      Case "zuendstufe":
        Zuendstufe = Wert
        Writeeeprom Zuendstufe , 2
        Print "Neue Zuendstufe: " ; Zuendstufe
      Case "brennstufe":
        Brennstufe = Wert
        Writeeeprom Brennstufe , 3
        Print "Neue Brennstufe: " ; Brennstufe
      Case "einwurfzeit":
        Einwurfzeit = Wert
        Writeeeprom Einwurfzeit , 4
        Print "Neue Einwurfzeit: " ; Einwurfzeit
      Case "tempkessel":
        Print Tempkessel
      Case "tempbrenner":
        Print Tempbrenner
      Case "zuendtemp":
        Zuendtemp = Wert
        Writeeeprom Zuendtemp , 5
        Print "Neue ZuendTemp: " ; Zuendtemp
      Case "brenntemp":
        Brenntemp = Wert
        Writeeeprom Brenntemp , 7
        Print "Neue BrennTemp: " ; Brenntemp
      Case "restarttemp":
        Restarttemp = Wert
        Writeeeprom Restarttemp , 9
        Print "Neue RestartTemp: " ; Restarttemp
      Case "minkesseltemp":
        Minkesseltemp = Wert
        Writeeeprom Minkesseltemp , 11
        Print "Neue MinKesselTemp: " ; Minkesseltemp
      Case "maxkesseltemp":
        Maxkesseltemp = Wert
        Writeeeprom Maxkesseltemp , 12
        Print "Neue MaxKesselTemp: " ; Maxkesseltemp
      Case "vref"
        Vref = Wert
        Writeeeprom Vref , 13
        Print "Neue Vref: " ; Vref
      Case "pellets"
        Pellets = Wert
        Writeeeprom Pellets , 17
        Print "Neue Pelletszeit: " ; Pellets
      Case "anlauf"
        Call Heizung_starten
      Case "start"
        'Betriebszustand = 1
        'Writeeeprom Betriebszustand , 9
        Print "Heizung an"

        Enable Timer0
        Enable Timer1
        Enable Interrupts

      Case "stop"
        'Betriebszustand = 0
        'Writeeeprom Betriebszustand , 9
        Print "Heizung stop"

        Pwm1a = 0
        Disable Timer0
        Disable Timer1
        Disable Interrupts
    End Select
  End If
End Sub

'Temperaturen von Brenner und Kessel einlesen
Sub Temperaturen_einlesen()
'PT100---------------------------------------------

  'Mittelung
  Tempdummy = 0
  For I = 1 To 10
    Adcwert = Getadc(1)
    Tempdummy = Tempdummy + Adcwert
    Waitms 10
  Next I
  Tempdummy = Tempdummy / 10

  'Reale Volt errechnen
  Tempdummy = Tempdummy * Vref
  Tempdummy = Tempdummy / 1024.0

  'Ohm errechnen
  Tempdummy = Tempdummy / 7                                 'Verstrkung weg
  Ptohm = Tempdummy / 0.005                                 'Ohm errechnen (durch strom teilen)

  'Korrektur
  Tempdummy = Ptohm * Ptohm
  Tempdummy = Tempdummy * Ptohm
  Tempdummy = Tempdummy * -0.00000567
  Tempkorrektur = Tempdummy


  Tempdummy = Ptohm * Ptohm
  Tempdummy = Tempdummy * 0.0024984
  Tempkorrektur = Tempkorrektur + Tempdummy

  Tempdummy = Ptohm * 2.22764
  Tempkorrektur = Tempkorrektur + Tempdummy

  Tempkorrektur = Tempkorrektur - 242.078

  'Gleitende Mittelung
  'Tempmittel = Tempraum * 4
  'Tempmittel = Tempmittel + Tempdummy
  'Tempmittel = Tempmittel / 5

  Tempkessel = Tempkorrektur
'  Print "O: " ; Ptohm
'  Print "Mt: " ; Temkessel
'PT100 Ende--------------------------------------


'Thermoelement-----------------------------------

  'Mittelung
  Tempdummy = 0
  For I = 1 To 10
    Adcwert = Getadc(0)
    Tempdummy = Tempdummy + Adcwert
    Waitms 10
  Next I
  Tempdummy = Tempdummy / 10

  'Reale Volt errechnen
  Tempdummy = Tempdummy * Vref
  Tempdummy = Tempdummy / 1024.0

  'Temperatur errechnen (5mV/C)
  Tempbrenner = Tempdummy / 0.005
'Thermoelement Ende------------------------------
End Sub


'Steuerung der Heizung
Sub Heizung_steuern
  'Brennt Heizung?
  If Heizungstabil = 1 Then

    'Pelletsbegrenzer runter zhlen
    If Pellets > 0 Then Decr Pellets

    'Brenntemperatur berwachen
    If Tempbrenner < Brenntemp And Pellets = 0 Then
      'Pellets nachfhren
      Set Schnecke
      Wait Einwurfzeit
      Reset Schnecke
      'Einwurf begrenzen
      Readeeprom Pellets , 17
    Elseif Tempbrenner < Restarttemp Then
      'Heizung neu starten
      Pwm1a = 0                                             'Geblse
      Reset Schnecke
      Reset Pumpe
      Reset Gluehkerze
      Call Heizung_starten
    End If

    'Kesseltemperatur berwachen
    If Tempkessel < Minkesseltemp Then
      'Pumpe einschalten
      Set Pumpe
      Call Heizung_starten
    Elseif Tempkessel > Maxkesseltemp Then
      'Pumpe ausschalten
      Reset Pumpe
      'HIER MUSS NOCH DIE HEIZUNG AUS GEMACHT WERDEN!!!-------------------------------------------------------------------------------------
      Pwm1a = 0

    End If

  End If
End Sub



'Timerinterruptziel fr Messung (Timer0 100Hz)
Timer0_irq:
  'Reloadwert setzen
  Timer0 = Timervorgabe0

  'Zhler erhhen, bei 100 ist eine Sekunde vergangen (100Hz * 100 = 1 Sekunde)
  Incr Timer0zaehler
  If Timer0zaehler = 100 Then

    'Zurcksetzten des Zhlers
    Timer0zaehler = 0

    'Temperaturen messen
    Call Temperaturen_einlesen

    'Regeln
    Call Heizung_steuern
  End If
Return