'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Autor: Jonas Eberhard www.joseng.de Projektarbeit EST 2010/2011 BKE09       |
'| Datei: IOTest.bas                                                           |
'| Beschreibung : IOTest der Heizungssteuerung. Zum ersten Testen der Platine  |
'|                ob alle Ein- und Ausgnge funktionieren                      |
'|                                                                             |
'| Version: 1.1                                                                |
'|                                                                             |
'| Fusebits: Lowbyte: 0xFF       Highbyte: 0xD9                                |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Befehle :                                                                   |
'|                                                                             |
'| - Gluehzeit : Zeit Zum "Vorglhen" Der Glhkerze                            |
'| - Zuendzeit : Zeit Zum Entznden Der Pellets Nach Ein Einwurf               |
'| - Zuendstufe : Geblsestufe Zum Znden Der Heizung [%]                      |
'| - Brennstufe : Geblsestufe Beim Brennen Der Heizung(normalbetrieb) [%]     |
'| - Einwurfzeit : Zeit Wie Lang Sich Die Pelletschnecke Dreht                 |
'| Die Zeit Bestimmt Die Menge Der Pellets                                     |
'| - Tempkessel : Gibt Die Kesseltemperatur Aus                                |
'| - Tempbrenner : Gibt Die Brennertemperatur Aus                              |
'| - Zuendtemp : Die Temperatur Ab Die Der Zndvorgang Abgeschlossen Ist       |
'|(die Heizung Brennt)                                                         |
'| - Brenntemp : Normaltemperatur Beim Brennen Der Heizung , Bei Unterschreiten|
'| Werden Pellets Nachgefhrt                                                  |
'| - Restarttemp : Die Temperatur Ab Die Die Heizung Neu Gestartet Wird        |
'| - Minkesseltemp : Minimale Kesseltemperatur                                 |
'| - Maxkesseltemp : Maximale Kesseltemperatur                                 |
'|                                                                             |
'|                                                                             |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Eeprom Belegung(byte Weise) :                                               |
'|                                                                             |
'| 0 : Glhzeit [s] (1 Byte)                                                   |
'| 1 : Zndzeit [s] (1 Byte)                                                   |
'| 2 : Zndstufe [%] (1 Byte)                                                  |
'| 3 : Brennstufe [%] (1 Byte)                                                 |
'| 4 : Einwurfzeit [s] (1 Byte)                                                |
'| 5 : Zndtemperatur [c] (2 Byte)                                            |
'| 7 : Brenntemperatur [c] (2 Byte)                                           |
'| 9 : Restarttemperatur [c] (2 Byte)                                         |
'| 11 : Minkesseltemp [c] (1 Byte)                                            |
'| 12 : Maxkesseltemp [c] (1 Byte)                                            |
'| 13: Vref [V] (4 Byte)                                                       |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Prinbelegung :                                                              |
'|                                                                             |
'|   Portc.0(adc0) : Temperaturelement(5mV / C)                               |
'|   Portc.1(adc1) : Pt100 Kessel(vin = 1ma * Rpt100)                          |
'|   Portd.0(rxd) : Serielle Schnittstelle Receive                             |
'|   Portd.1(txd) : Serielle Schnittstelle Transmit                            |
'|   Portd.3 : Richtungspin Rs485 Baustein(busrichtung)                        |
'|   Portd.5 : Pumpe                                                           |
'|   Portd.6 : Schnecke                                                        |
'|   Portd.7 : Glhkerze                                                       |
'|   Portb.1 : Geblse                                                         |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Kommandoaufbau :                                                            |
'|                                                                             |
'|   Kommando : "Modulname: Befehl: Wert"                                      |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'Init
$regfile = "m8def.dat"                                      'ATMega 8
$crystal = 16000000                                         '16MHz Quarz
$baud = 19200                                               'Baudrate 19200

$hwstack = 40
$swstack = 40
$framesize = 40


'Funktionsprototypen
Declare Sub Heizung_starten()
Declare Sub Temperaturen_einlesen()
Declare Sub Interpreter()
Declare Sub Heizung_steuern()


'Ein-/Ausgnge
Config Portd.3 = Output                                     'Busrichtung    0= Lesen 1=Schreiben
Config Portb.1 = Output                                     'Geblse
Config Portd.6 = Output                                     'Motor Schnecke
Config Portd.5 = Output                                     'Pumpe
Config Portd.7 = Output                                     'Trigger Glhkerze


'Aliaszuweisungen
Busrichtung Alias Portd.3
Geblaese Alias Portb.1
Schnecke Alias Portd.6
Pumpe Alias Portd.5
Gluehkerze Alias Portd.7


'Timer0 (100H)
Config Timer0 = Timer , Prescale = 1024
On Timer0 Timer0_irq
Enable Timer0


'Timer1 (1Hz)
Config Timer1 = Pwm , Pwm = 10 , Compare A Pwm = Clear Down , Compare B Pwm = Clear Down , Prescale = 1
'On Timer1 Timer1_irq
Enable Timer1


'Deklarationen
Const Timervorgabe0 = 100                                   'Reloadvalue fr PWMTimer (Timer0)
Dim I As Byte                                               'Zhlvariable
Dim Timer0zaehler As Byte                                   'Timer0 zhler

'Startwerte
Reset Schnecke                                              'Alles aus
Reset Pumpe
Reset Gluehkerze
Pwm1a = 0                                                   'Geblaese

Timer0 = Timervorgabe0
'Timer1 = Timervorgabe1

Enable Interrupts


'Hauptprogramm
Do
  'PWM rauf/runter fahren
  For I = 1 To 100
    Pwm1a = I * 10
    Waitms 10
  Next I

  For I = 100 To 1 Step -1
    Pwm1a = I * 10
    Waitms 10
  Next I
Loop


'Timerinterruptziel fr Messung (Timer0 100Hz)
Timer0_irq:
  'Reloadwert setzen
  Timer0 = Timervorgabe0

  'Zhler erhhen, bei 100 ist eine Sekunde vergangen (100Hz * 100 = 1 Sekunde)
  Incr Timer0zaehler
  If Timer0zaehler = 100 Then

    'Zurcksetzten des Zhlers
    Timer0zaehler = 0

    Toggle Schnecke                                         'Alles aus
    Toggle Pumpe
    Toggle Gluehkerze
    Toggle Busrichtung

  End If
Return