'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Autor: Jonas Eberhard www.joseng.de Projektarbeit EST 2010/2011 BKE09       |
'| Datei: Heizungssteuerung.bas                                                |
'| Beschreibung : Firmware der Heizungssteuerung (zustndig fr Brenn -        |
'|                 berwachung, Pelletnachfhrung und Kesseltemperatur         |
'|                                                                             |
'| Version: 1.0                                                                |
'|                                                                             |
'| Fusebits: Lowbyte: 0xFF       Highbyte: 0xD1                                |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Befehle :                                                                   |
'|                                                                             |
'| - Gluehzeit : Zeit Zum "Vorglhen" Der Glhkerze                            |
'| - Zuendzeit : Zeit Zum Entznden Der Pellets Nach Ein Einwurf               |
'| - Zuendstufe : Geblsestufe Zum Znden Der Heizung [%]                      |
'| - Brennstufe : Geblsestufe Beim Brennen Der Heizung(normalbetrieb) [%]     |
'| - Einwurfzeit : Zeit Wie Lang Sich Die Pelletschnecke Dreht                 |
'| Die Zeit Bestimmt Die Menge Der Pellets                                     |
'| - Tempkessel : Gibt Die Kesseltemperatur Aus                                |
'| - Tempbrenner : Gibt Die Brennertemperatur Aus                              |
'| - Zuendtemp : Die Temperatur Ab Die Der Zndvorgang Abgeschlossen Ist       |
'|(die Heizung Brennt)                                                         |
'| - Brenntemp : Normaltemperatur Beim Brennen Der Heizung , Bei Unterschreiten|
'| Werden Pellets Nachgefhrt                                                  |
'| - Restarttemp : Die Temperatur Ab Die Die Heizung Neu Gestartet Wird        |
'| - Minkesseltemp : Minimale Kesseltemperatur                                 |
'| - Maxkesseltemp : Maximale Kesseltemperatur                                 |
'|                                                                             |
'|                                                                             |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Eeprom Belegung(byte Weise) :                                               |
'|                                                                             |
'| 0 : Glhzeit [s] (1 Byte)                                                   |
'| 1 : Zndzeit [s] (1 Byte)                                                   |
'| 2 : Zndstufe [%] (1 Byte)                                                  |
'| 3 : Brennstufe [%] (1 Byte)                                                 |
'| 4 : Einwurfzeit [s] (1 Byte)                                                |
'| 5 : Zndtemperatur [c] (2 Byte)                                            |
'| 7 : Brenntemperatur [c] (2 Byte)                                           |
'| 9 : Restarttemperatur [c] (2 Byte)                                         |
'| 11 : Minkesseltemp [c] (1 Byte)                                            |
'| 12 : Maxkesseltemp [c] (1 Byte)                                            |
'| 13: Vref [V] (4 Byte)                                                       |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Prinbelegung :                                                              |
'|                                                                             |
'|   Portc.0(adc0) : Temperaturelement(5mV / C)                               |
'|   Portc.1(adc1) : Pt100 Kessel(vin = 1ma * Rpt100)                          |
'|   Portd.0(rxd) : Serielle Schnittstelle Receive                             |
'|   Portd.1(txd) : Serielle Schnittstelle Transmit                            |
'|   Portd.3 : Richtungspin Rs485 Baustein(busrichtung)                        |
'|   Portd.5 : Pumpe                                                           |
'|   Portd.6 : Schnecke                                                        |
'|   Portd.7 : Glhkerze                                                       |
'|   Portb.1 : Geblse                                                         |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Kommandoaufbau :                                                            |
'|                                                                             |
'|   Kommando : "Modulname: Befehl: Wert"                                      |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'Init
$regfile = "m8def.dat"                                      'ATMega 8
$crystal = 16000000                                         '16MHz Quarz
$baud = 19200                                               'Baudrate 19200

$hwstack = 40
$swstack = 40
$framesize = 40

$projecttime = 0


'Funktionsprototypen
Declare Sub Temperaturen_einlesen()
Declare Sub Interpreter()


'Ein-/Ausgnge
Config Portd.3 = Output                                     'Busrichtung    0= Lesen 1=Schreiben
Config Portb.1 = Output                                     'Geblse
Config Portd.6 = Output                                     'Motor Schnecke
Config Portd.5 = Output                                     'Pumpe
Config Portd.7 = Output                                     'Trigger Glhkerze


'Aliaszuweisungen
Busrichtung Alias Portd.3
Geblaese Alias Portb.1
Schnecke Alias Portd.6
Pumpe Alias Portd.5
Gluehkerze Alias Portd.7


'ADC
Config Adc = Single , Prescaler = Auto
'ADC Kanal 0 und Kanal 1 sind belegt.
'Kanal 0: Temp Element (5mV/C)
'Kanal 1: PT100


'Timer0 (100H)
Config Timer0 = Timer , Prescale = 1024
On Timer0 Timer0_irq
Enable Timer0


'Timer1 (PWM)
Config Timer1 = Pwm , Pwm = 10 , Compare A Pwm = Clear Down , Compare B Pwm = Clear Down , Prescale = 1
'On Timer1 Timer1_irq
Enable Timer1


'Serielle Schnittstelle RS485
Config Print0 = Portd.3 , Mode = Set


'Deklarationen
Dim Tempbrenner As Integer                                  'Temperatur Brenner [C]
Dim Tempkessel As Integer                                   'Temperatur Kessel [C]
Dim Tempdummy As Single                                     'Rechenvariable fr PT100 Korrektur
Dim Adcwert As Integer                                      'Eingelesener Wert des ADCs
Dim I As Integer                                            'Zhlervariable fr mittelung
Dim Tempkorrektur As Single                                 'Rechenvariable zur Korrektur des PT100
Dim Ptohm As Single                                         'Widerstand des PT100
Dim Vref As Single                                          'Referenzspannung der ADCs = 5V
Dim Modulname As String * 10                                'Modulname fr Bus
Const Timervorgabe0 = 100                                   'Reloadvalue fr PWMTimer (Timer0)
Dim Buffer As String * 50                                   'Serieller Buffer
Dim Laenge As Byte                                          'Lnge eines Strings (wird universal eingesetzt)
Dim Kommando As String * 15                                 'Aktuelles Kommanddo von Bus
Dim Wert As Integer                                         'Parameter zu Kommando
Dim Modul As String * 10                                    'Name des angesprochenen Moduls
Dim Timer0zaehler As Byte                                   'Zhler fr 1 Sekunde


'Startwerte
Reset Schnecke                                              'Alles aus
Reset Pumpe
Reset Gluehkerze
Pwm1a = 0                                                   'Geblaese

Vref = 5.09
Modulname = "Heizung: "

Timer0 = Timervorgabe0
'Timer1 = Timervorgabe1

Enable Interrupts


'Hauptprogramm
Do
  Input ">" , Buffer
  Call Interpreter
Loop


'Befehlinterpreter fr Buskommandos
Sub Interpreter()
  Laenge = Len(modulname)
  Modul = Left(buffer , Laenge)
  If Modul = Modulname Then
    'Seriellen String zerlegen
    'Kommando
    Laenge = Len(buffer) - Laenge
    Buffer = Right(buffer , Laenge)
    Laenge = Instr(buffer , " ")
    Laenge = Laenge - 1
    Kommando = Left(buffer , Laenge)
    Kommando = Lcase(kommando)

    'Wert
    Laenge = Len(buffer) - Laenge
    Laenge = Laenge - 1
    Buffer = Right(buffer , Laenge)
    Wert = Val(buffer)

    'Debugausgabe
    Print "M " ; Modul
    Print "K " ; Kommando
    Print "W " ; Wert

  End If
End Sub


'Temperaturen von Brenner und Kessel einlesen
Sub Temperaturen_einlesen()
'PT100---------------------------------------------

  'Mittelung
  Tempdummy = 0
  For I = 1 To 10
    Adcwert = Getadc(1)
    Tempdummy = Tempdummy + Adcwert
    Waitms 10
  Next I
  Tempdummy = Tempdummy / 10

Print "T1: " ; Tempdummy

  'Reale Volt errechnen
  Tempdummy = Tempdummy * Vref
  Tempdummy = Tempdummy / 1024.0

  'Ohm errechnen
  Tempdummy = Tempdummy / 7                                 'Verstrkung weg
  Ptohm = Tempdummy / 0.005                                 'Ohm errechnen (durch strom teilen)

Print "T2: " ; Tempdummy

  'Korrektur
  Tempdummy = Ptohm * Ptohm
  Tempdummy = Tempdummy * Ptohm
  Tempdummy = Tempdummy * -0.00000567
  Tempkorrektur = Tempdummy


  Tempdummy = Ptohm * Ptohm
  Tempdummy = Tempdummy * 0.0024984
  Tempkorrektur = Tempkorrektur + Tempdummy

  Tempdummy = Ptohm * 2.22764
  Tempkorrektur = Tempkorrektur + Tempdummy

  Tempkorrektur = Tempkorrektur - 242.078

  'Gleitende Mittelung
  'Tempmittel = Tempraum * 4
  'Tempmittel = Tempmittel + Tempdummy
  'Tempmittel = Tempmittel / 5

  Tempkessel = Tempkorrektur
  Print "O: " ; Ptohm
  Print "Mt: " ; Tempkessel
'PT100 Ende--------------------------------------


'Thermoelement-----------------------------------

  'Mittelung
  Tempdummy = 0
  For I = 1 To 10
    Adcwert = Getadc(0)
    Tempdummy = Tempdummy + Adcwert
    Waitms 10
  Next I
  Tempdummy = Tempdummy / 10

  'Reale Volt errechnen
  Tempdummy = Tempdummy * Vref
  Tempdummy = Tempdummy / 1024.0

  'Temperatur errechnen (5mV/C)
  Tempbrenner = Tempdummy / 0.005
'Thermoelement Ende------------------------------
End Sub


'Timerinterruptziel fr Messung (Timer0 100Hz)
Timer0_irq:
  'Reloadwert setzen
  Timer0 = Timervorgabe0

  'Zhler erhhen, bei 100 ist eine Sekunde vergangen (100Hz * 100 = 1 Sekunde)
  Incr Timer0zaehler
  If Timer0zaehler = 100 Then

    'Zurcksetzten des Zhlers
    Timer0zaehler = 0

    'Temperaturen messen
    Call Temperaturen_einlesen

    'Zum Test Temperaturen ausgeben
    Print "Tempkessel: " ; Tempkessel
    Print "Tempbrenner: " ; Tempbrenner

    'Pins bedienen (Lebenszeichen ;)
    Toggle Schnecke
    Toggle Gluehkerze
    Toggle Pumpe
  End If
Return