'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Autor: Jonas Eberhard www.joseng.de Projektarbeit EST 2010/2011 BKE09       |
'| Datei: Regler.bas                                                           |
'| Beschreibung : Firmware des Temperautrreglers                               |
'|                                                                             |
'| Fusebits: Lowbyte: 0xFF       Highbyte: 0xD9                                |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Befehle :                                                                   |
'|  - Tempraum: Gibt die aktuelle Raumtemperatur aus                           |
'|  - Solltemp: Gibt die Tolltemperatur fr den Raum an                        |
'|  - Hysterese: Hysterese fr Temperaturregelung                              |
'|  - Reglertyp: Stellt den Reglertyp ein (2-Punktregler/P-Regler)             |
'|  - Kfaktor: Stellt den Verstrkungsfaktor fr P-Regler ein                  |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Eeprom Belegung(byte Weise) :                                               |
'|  0: Solltemperatur fr Raum                                                 |
'|  1: Hysterese fr Regler                                                    |
'|  2: Reglertyp                                                               |
'|  3: Kfaktor (fr P-Regler)                                                  |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Prinbelegung :                                                              |
'|                                                                             |
'|   Portd.3 Busrichtung                                                       |
'|   Portb.0 Pumpe                                                             |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Kommandoaufbau :                                                            |
'|                                                                             |
'|   Kommando : "Modulname: Befehl: Wert"                                      |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'Init
$regfile = "m8def.dat"                                      'ATMega 8
$crystal = 16000000                                         '16MHz Quarz
$baud = 19200                                               'Baudrate 19200


'Ein-/Ausgnge
Config Portd.3 = Output                                     'Busrichtung    0= Lesen 1=Schreiben
Config Portb.0 = Output


'Aliaszuweisungen
Busrichtung Alias Portd.3
Pumpe Alias Portb.0


'Timer0 (20kH)
Config Timer0 = Timer , Prescale = 8
On Timer0 Pwm
Enable Timer0


'Timer1 (1Hz)
Config Timer1 = Timer , Prescale = 256
On Timer1 Timer1_irq
Enable Timer1


'Deklarationen
Const Timervorgabe0 = 156                                   'Reloadvalue fr PWMTimer (Timer0)
Const Timervorgabe1 = 3036                                  'Reloadvalue fr Messtimer (Timer1)
Dim Pwmcount As Integer                                     'Zhlvariable fr PWM
Dim Pwmcalc As Integer                                      'Rechenvariable fr PWM
Const Minpuls = 2                                           'Minimale Counts fr PWM Puls, damit der PWM Puls nicht zu kurz wird, kann Motor schaden
Dim Pumpenstufe As Byte                                     'Gibt/Setzt die Pumpenleistung (0-100%)


'Startwerte
Timer0 = Timervorgabe0
Timer1 = Timervorgabe1

Reset Pumpe
Pumpenstufe = 0                                             'Pumpe ausschalten

Enable Interrupts


'Hauptprogramm
Do
  For Pumpenstufe = 1 To 100
    Waitms 100
  Next Pumpenstufe

  For Pumpenstufe = 100 To 1 Step -1
    Waitms 100
  Next Pumpenstufe
Loop


'Timerinterruptziel fr PWM (Timer0 20kH)
Pwm:
  'Reloadwert setzten
  Timer0 = Timervorgabe0

  'Auflsung 50S
  'PWM-Frequenz 10mS = 100Hz

  'Werte errechnen
  Pwmcount = Pwmcount + 1
  If Pumpenstufe > 100 Then Pumpenstufe = 100
  Pwmcalc = 2 * Pumpenstufe

  'Wert erreicht dann Pin lschen
  If Pwmcount = Pwmcalc Then Reset Pumpe


  'Sind 10mS vorbei...
  If Pwmcount = 200 Then
    '...dann zhler lschen
    Pwmcount = 0
    '...und pin setzten
    If Pumpenstufe > 0 Then Set Pumpe
  End If
Return


'Timerinterruptziel fr Messung (Timer1 1Hz)
Timer1_irq:
  'Reloadwert setzen
  Timer1 = Timervorgabe1


Return