'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Autor: Jonas Eberhard www.joseng.de Projektarbeit EST 2010/2011 BKE09       |
'| Datei: Regler.bas                                                           |
'| Beschreibung : Firmware des Temperautrreglers                               |
'|                                                                             |
'| Version: 1.3                                                                |
'|                                                                             |
'| Fusebits: Lowbyte: 0xFF       Highbyte: 0xD9                                |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Befehle :                                                                   |
'|  - Tempraum: Gibt die aktuelle Raumtemperatur aus                           |
'|  - Solltemp: Gibt die Tolltemperatur fr den Raum an                        |
'|  - Hysterese: Hysterese fr Temperaturregelung                              |
'|  - Reglertyp: Stellt den Reglertyp ein (1: 2-Punktregler/2: P-Regler)       |
'|  - Kfaktor: Stellt den Verstrkungsfaktor fr P-Regler ein                  |
'|  - Vref: Referenzspannung des ADCs                                          |
'|  - Start: Startet den Reglerbetrieb                                         |
'|  - Stop: Stoppt den Reglerbetrieb bis zum Befehl Start                      |
'|                                                                             |
'|  Die Gro/Kleinschreibweise der Befehle ist egal, beim Modulname nicht!     |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Eeprom Belegung(byte Weise) :                                               |
'|  1: Solltemperatur fr Raum (1 Byte)                                        |
'|  2: Hysterese fr Regler (1 Byte)                                           |
'|  3: Reglertyp (1 Byte)                                                      |
'|  4: Kfaktor (fr P-Regler) (1 Byte)                                         |
'|  5: Vref (4 Byte)                                                           |
'|  9: Betriebszustand (Start/Stop) (1 Byte)                                   |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Prinbelegung :                                                              |
'|                                                                             |
'|   Portd.3 Busrichtung                                                       |
'|   Portb.0 Pumpe                                                             |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Kommandoaufbau :                                                            |
'|                                                                             |
'|   Kommando : "Modulname: Befehl Wert"                                       |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'Init
$regfile = "m8def.dat"                                      'ATMega 8
$crystal = 16000000                                         '16MHz Quarz
$baud = 19200                                               'Baudrate 19200

$hwstack = 40
$swstack = 40
$framesize = 40


'Subs/Functions
Declare Sub Temperaturen_einlesen()
Declare Sub Interpreter()
Declare Sub Regler()


'Ein-/Ausgnge
Config Portd.3 = Output                                     'Busrichtung    0= Lesen 1=Schreiben
'Config Portb.0 = Output


'Aliaszuweisungen
Busrichtung Alias Portd.3
'Pumpe Alias Portb.0


'ADC
Config Adc = Single , Prescaler = Auto
'Kanal 0: PT100


'Timer0 (100H)
Config Timer0 = Timer , Prescale = 1024
On Timer0 Timer0_irq
Enable Timer0



'Timer1 (PWM)
Config Timer1 = Pwm , Pwm = 10 , Compare A Pwm = Clear Down , Compare B Pwm = Clear Down , Prescale = 1
Enable Timer1


'Serielle Schnittstelle RS485
Config Print = Portd.3 , Mode = Set

'Serielle Schnittstelle Interrupt
'On Urxc Rec_isr
'Enable Urxc


'Deklarationen
Const Timervorgabe0 = 100                                   'Reloadvalue fr PWMTimer (Timer0)
Const Timervorgabe1 = 3036                                  'Reloadvalue fr Messtimer (Timer1)
Dim Adcwert As Integer                                      'Eingelesener Wert von ADC
Dim Tempdummy As Single                                     'rechenvariable fr pt100 korrektur
Dim Tempkorrektur As Single
Dim Ptohm As Single                                         'Widerstand des PT100
Dim Vref As Single                                          'Referenzspannung der ADCs = 5V
Dim Tempraum As Integer                                     'Aktuelle Raumtemperatur
Dim I As Integer                                            'zhlervariable fr mittelung
Dim Tempmittel As Single                                    'Rechenvariable fr gleitende Mittelwertsbildung
Dim Buffer As String * 50                                   'Serieller Buffer
Dim Laenge As Byte                                          'Lnge eines Strings (wird universal eingesetzt)
Dim Char As Byte                                            'Aktuelles Serielles Zeichen
Dim Kommando As String * 15                                 'Aktuelles Kommanddo von Bus
Dim Wert As Single                                          'Parameter zu Kommando
Dim Modulname As String * 9                                 'Modulname fr Bus
Dim Pumpenstufe As Integer                                  'Gibt/Setzt die Pumpenleistung (0-100%)
Dim Messung As Byte                                         'Freigabevariable fr Messung und Regelung
Dim Reglertyp As Byte                                       'Aktiver Reglertyp 2-Punkt/P-Regler
Dim Solltemp As Byte                                        'Gibt die Raumsolltemperatur an
Dim Differenz As Integer                                    'Differenz bei Temperaturregler
Dim Hysterese As Byte                                       'Reglerhysterese
Dim Kfaktor As Byte                                         'Verstrkungsfaktor fr P-Regler
Dim Betriebszustand As Byte                                 'gibt des Betriebszustand an (start/stop)
Dim Modul As String * 10                                    'Name des angesprochenen Moduls
Dim Timer0zaehler As Byte                                   'Zhler fr 1 Sekunde


'Einstellungen aus EEprom laden
Readeeprom Solltemp , 1
Readeeprom Hysterese , 2
Readeeprom Reglertyp , 3
Readeeprom Kfaktor , 4
Readeeprom Vref , 5
Readeeprom Betriebszustand , 9


'Startwerte
Modulname = "Regler: "                                      'Name des Moduls
Timer0 = Timervorgabe0
Timer1 = Timervorgabe1
Pwm1a = 0                                                   'Pumpe ausschalten
Tempraum = 0

Enable Interrupts


'Wenn Regler gestoppt dann alles aus
If Betriebszustand = 0 Then
  Reglertyp = 0
  Pwm1a = 0
  Disable Interrupts
End If


'Hauptprogramm
Do
  Input Buffer Noecho
  Call Interpreter
Loop


'Temperaturen von Brenner und Kessel einlesen
Sub Temperaturen_einlesen()

  'Mittelung
  Tempdummy = 0
  For I = 1 To 10
    Adcwert = Getadc(0)
    Tempdummy = Tempdummy + Adcwert
    Waitms 10
  Next I
  Tempdummy = Tempdummy / 10

  'Reale Volt errechnen
  Tempdummy = Tempdummy * Vref
  Tempdummy = Tempdummy / 1024.0

  'Offset abziehen  (Offset = 5mA * 100Ohm * 7)
  'Tempdummy = Tempdummy - 3.5

  'Ohm errechnen
  Tempdummy = Tempdummy / 7                                 'Verstrkung weg
  Ptohm = Tempdummy / 0.005                                 'ohm errechnen (durch strom teilen)

  'Korrektur
  Tempdummy = Ptohm * Ptohm
  Tempdummy = Tempdummy * Ptohm
  Tempdummy = Tempdummy * -0.00000567
  Tempkorrektur = Tempdummy


  Tempdummy = Ptohm * Ptohm
  Tempdummy = Tempdummy * 0.0024984
  Tempkorrektur = Tempkorrektur + Tempdummy

  Tempdummy = Ptohm * 2.22764
  Tempkorrektur = Tempkorrektur + Tempdummy

  Tempkorrektur = Tempkorrektur - 242.078


  'PT100 Korrektur 0,003850
  'Tempdummy = Tempdummy / 0.00385

  'Gleitende Mittelung
  'Tempmittel = Tempraum * 4
  'Tempmittel = Tempmittel + Tempdummy
  'Tempmittel = Tempmittel / 5

  Tempraum = Tempkorrektur
'  Print "O: " ; Ptohm
'  Print "Mt: " ; Tempraum
End Sub


'Befehlinterpreter fr Buskommandos
Sub Interpreter()
  Laenge = Len(modulname)
  Modul = Left(buffer , Laenge)
  If Modul = Modulname Then
    'Seriellen String zerlegen
    'Kommando
    Laenge = Len(buffer) - Laenge
    Buffer = Right(buffer , Laenge)
    Laenge = Instr(buffer , " ")
    Laenge = Laenge - 1
    Kommando = Left(buffer , Laenge)
    Kommando = Lcase(kommando)

    'Wert
    Laenge = Len(buffer) - Laenge
    Laenge = Laenge - 1
    Buffer = Right(buffer , Laenge)
    Wert = Val(buffer)

'    'Debugausgabe
'    Print "M " ; Modul
'    Print "K " ; Kommando
'    Print "W " ; Wert


    'Kommando auswerten

    'Der Ablauf eines Befehls ist immer der selbe:
    'Erst wird geprft ob der Parameter des Befehls ein ? war,
    'wenn ja, dann wird dessen Wert zurckgegeben,
    'sonst wird der neue Wert gespeichert und mit einer
    'Meldung mit neuem Wert quitiert

    Select Case Kommando
      Case "tempraum"                                       'Tempraum
        Print Tempraum

      Case "solltemp"                                       'Solltemp
        If Buffer = "?" Then
          Print Solltemp
          Exit Sub
        End If
        Solltemp = Wert
        Writeeeprom Solltemp , 1
        Print "Neue Solltemp: " ; Solltemp

      Case "hysterese"                                      'Hysterese
        If Buffer = "?" Then
          Print Hysterese
          Exit Sub
        End If
        Hysterese = Wert
        Writeeeprom Hysterese , 2
        Print "Neue Hysterese: " ; Hysterese

      Case "reglertyp"                                      'Reglertyp
        If Buffer = "?" Then
          Print Reglertyp
          Exit Sub
        End If
        Reglertyp = Wert
        Writeeeprom Reglertyp , 3
        Print "Reglertyp ist: " ; Reglertyp

      Case "kfaktor"                                        'Kfaktor
        If Buffer = "?" Then
          Print Kfaktor
          Exit Sub
        End If
        Kfaktor = Wert
        Writeeeprom Kfaktor , 4
        Print "Neuer Kfaktor: " ; Kfaktor

      Case "vref"                                           'Vref
        If Buffer = "?" Then
          Print Vref
          Exit Sub
        End If
        Vref = Wert
        Writeeeprom Vref , 5
        Print "Neue Vref: " ; Vref

      Case "start"                                          'Start
        Betriebszustand = 1
        Writeeeprom Betriebszustand , 9
        Print "Regler start"

        Readeeprom Reglertyp , 3
        Enable Timer0
        Enable Timer1
        Enable Interrupts

      Case "stop"                                           'Stop
        Betriebszustand = 0
        Writeeeprom Betriebszustand , 9
        Print "Regler stop"

        Reglertyp = 0
        Pwm1a = 0
        Disable Timer0
        Disable Timer1
        Disable Interrupts
    End Select
  End If
  Buffer = ""
End Sub


'Timerinterruptziel fr Messung (Timer0 100Hz)
Timer0_irq:
  'Reloadwert setzen
  Timer0 = Timervorgabe0

  'Zhler erhhen, bei 100 ist eine Sekunde vergangen (100Hz * 100 = 1 Sekunde)
  Incr Timer0zaehler
  If Timer0zaehler = 100 Then

    'Zurcksetzten des Zhlers
    Timer0zaehler = 0

    'Temperaturen messen
    Call Temperaturen_einlesen

    'Regeln
    Call Regler
  End If
Return


'Reglerroutinen
Sub Regler()
  'Zweipunktregler
  If Reglertyp = 1 Then

    'Oberen Bereich (ber Solltemperatur) errechnen und vergleichen => Pumpe aus
    'Der Name Differenz ist hier eigendlich falsch, da es keine Differenz ist, sondern eine Vegleichstemperatur
    '  die Variable wird verwendet um RAM zu sparen
    Differenz = Hysterese / 2
    Differenz = Differenz + Solltemp
'    Print "R1d: " ; Differenz
    If Tempraum >= Differenz Then
      Pwm1a = 0
    End If

    'Unteren Bereich (unter Solltemperatur) errechnen und Vergleichen => Pumpe an
    'Hier gilt das gleiche fr Differenz wie oben!
    Differenz = Hysterese / 2
    Differenz = Differenz * -1
    Differenz = Differenz + Solltemp
'    Print "R1dn: " ; Differenz
    If Tempraum < Differenz Then
      Pwm1a = 1023
    End If

'    Print "R1ps: " ; PWM1A
  End If

  'P-Regler
  If Reglertyp = 2 Then

    'Regeldifferenz errechnen
    Differenz = Solltemp - Tempraum
'    Print "R2d: " ; Differenz

    'Abfangen einer negativen Differenz => Gemessene Temperatur hher als Solltemp
    If Differenz < 0 Then Differenz = 0

    'PWM Wert errechnen
    Pumpenstufe = Differenz * Kfaktor

    'Abfrage ob Wert zu hoch, da sonst berluft und Wert sich verndert
    '1023 ist der maximale Wert bei 10bit
    If Pumpenstufe > 1023 Then Pumpenstufe = 1023
    Pwm1a = Pumpenstufe
'    Print "R2ps: " ; Pumpenstufe

  End If
'  Print "PS: " ; Pwm1a
End Sub